#include "sgial.h"

/*
 * Initialize the plugin
 */
void sgial_init(void)
{
  /*
   * These are for the al* calls
   */
  ALparamInfo gaininfo;
  ALpv pv[1], qualifiers[1];

  /* 
   * AL_MAX_SETSIZE is supposed to be a 32 bit int
   */
  gint32 setsize;

  /*
   * Open up the config file and read in the parameters
   */
  filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
  if((cfgfile = xmms_cfg_open_file(filename))) {
    if(!xmms_cfg_read_string(cfgfile, "SGIAL", "output_device",
			     &alinfo.current_device)) {
      alinfo.current_device = g_strdup("AnalogOut");
    }
    if(!xmms_cfg_read_int(cfgfile, "SGIAL", "buffer_size", &alinfo.bufsize))
      alinfo.bufsize = 10000;
    xmms_cfg_free(cfgfile);
  }

  /* 
   * Find the resource that corresponds to alinfo.current_device
   */
  alinfo.current_dev = alGetResourceByName(AL_SYSTEM, alinfo.current_device, 
					   AL_OUTPUT_DEVICE_TYPE);
  
  if (!alinfo.current_dev) {
    fprintf(stderr,"Invalid audio resource: %s (%s)\n", alinfo.current_device,
            alGetErrorString(oserror()));
    return;
  }
  
  /*
   * Allocate enough room to store the list of output devices
   */
  pv[0].param = AL_MAX_SETSIZE;
  alGetParams(AL_SYSTEM, pv, 1);
  setsize = pv[0].value.i;
  alinfo.output_devices = (ALvalue *)g_malloc(setsize * sizeof(ALvalue));

  /*
   * Get a list of all output devices for config dialog
   */
  qualifiers[0].param = AL_TYPE;
  qualifiers[0].value.i = AL_OUTPUT_DEVICE_TYPE;
  if ( (alinfo.device_count = alQueryValues(AL_SYSTEM, AL_DEVICES, 
					   alinfo.output_devices,
					   setsize, qualifiers, 1)) < 0) {
    fprintf(stderr, "alQueryValues failed: %s\n", 
	    alGetErrorString(oserror()));
  }

  /*
   * Get info about the volume
   * Returns min and max in decibels
   */
  if (alGetParamInfo(alinfo.current_dev, AL_GAIN, &gaininfo) < 0) {
    fprintf(stderr, "init: alGetParamInfo failed: %s\n", 
	    alGetErrorString(oserror()));
  }

  /*
   * Just in case the minimum gain is < 0, we shift the number
   * line up to 0 as a base to make it easy to calculate the volume
   * in %.  If the minimum gain is 0, the adjustment we add in 
   * get_volume and set_volume will be 0, which will make things
   * easy.  Otherwise, we'll add alinfo.adjustment to what we get
   * from the audio library in get_volume and divide by alinfo.gainrange
   * to get the %, and subtract it from the result of our calculation
   * in set_volume.
   */
  alinfo.adjusted_maxgain = alFixedToDouble(gaininfo.max.ll);
  alinfo.adjusted_mingain = alFixedToDouble(gaininfo.min.ll);
  alinfo.adjustment = MAX(-alinfo.adjusted_mingain, alinfo.adjusted_mingain);
  alinfo.adjusted_mingain = 0;
  alinfo.gainrange = alinfo.adjusted_maxgain + alinfo.adjustment;


  /*
   * Initialize some globals
   */
  alinfo.port = NULL;
  pinfo.written_frames = 0;
  pinfo.output_frames = 0;

  return;
}

